<?php

  /* XXX: Show links to other objects.  Don't show cities by default. */

  if ($_POST['city_name']) {
    $id = add_city($_POST['city_name'], $_POST['city_postcode_area']);
    if ($id !== false) {
      echo "<p>Added city.</p>\n";
      $parameters = array($_POST['city_name'], $id);
    }
  }

  function show_cities($name = null) {
    list($first_page, $per_page) = pagination();
    echo "<p>Cities:";
    $q = new CityQuery;
    if (isset($name)) $q->filterByName($name);
    $cities = $q->paginate($first_page, $per_page);
    if (count($cities)) {
      foreach ($cities as $city) {
        echo "<br>\nCity: " . $city->getStrongLink(get_city_displayname($city));
        $n = $city->getName();
        $i = $city->getId();
        echo " " . get_small_link("Areas", "/area/in/city/%s/%d", $n, $i);
        echo " " . get_small_link("Contacts", "/contact/in/city/%s/%d", $n, $i);
        echo " " . get_small_link("Donations", "/donation/in/city/%s/%d", $n, $i);
        echo " " . get_small_link("Orders", "/order/in/city/%s/%d", $n, $i);
        if (check_admin(1)) {
          echo " " . $city->getDeleteLink();
        }
      }
      show_pagination($cities);
    }
    else echo " none";
    echo "</p>\n";
  }

  function show_new_city_form() {
    if (! check_admin(1)) return;

    form("noprint standout");
    echo "<p>Add a new city <input name=\"city_name\">\n";
    echo "with postcode prefix <input name=\"city_postcode_area\" size=4 maxlength=4>\n";
    echo "<input type=\"submit\" value=\"Add\"></p>\n";
  }

  function add_city($name, $postcode) {
    if (! check_admin(1, "add a city")) return;

    if (preg_match('/^([A-Za-z]+)/', $postcode, $m)) {
      $prefix = strtoupper($m[1]);
    }
    else {
      echo "<p>Invalid postcode prefix!</p>\n";
      return false;
    }

    $city = get_city_by_name($name, $prefix, false);
    if ($city) {
      echo "<p>$name already exists!</p>\n";
      show_city($name, $city->getId());
      return false;
    }

    $city = new City;
    $city->setName($name);
    $city->setPostcodeArea($prefix);

    try {
      $city->save();
    }
    catch (Exception $e) {
      echo "<p>Error adding $name!</p>\n";
      /* XXX: Why? */
      return false;
    }

    return $city->getId();
  }

  function confirm_delete_city($name, $id = null) {
    if (! check_admin(1, "delete a city")) return;

    if (isset($id)) $city = get_city_by_id($id);
    else $city = get_city_by_name($name);
    if (! $city) return false;

    echo "<h3>Confirm deletion</h3>\n";
    echo "<p>You must confirm deletion of city " . $city->getName() . ": " . $city->getDeleteLink(true) . "</p>\n";
  }

  function delete_city($name, $id = null) {
    if (! check_admin(1, "delete a city")) return;

    if (isset($id)) $city = get_city_by_id($id);
    else $city = get_city_by_name($name);
    if (! $city) return false;

    try {
      $city->delete();
      echo "<p>Deleted city.</p>\n";
    }
    catch (Exception $e) {
      echo "<p>Error deleting $name!</p>\n";
      /* XXX: Why? Check for addresses in use... */
      return false;
    }

    return true;
  }

  function show_city($name, $id = null) {
    if (isset($id)) $city = get_city_by_id($id);
    else $city = get_city_by_name($name);
    if (! $city) return;

    echo "<p>City: <span class=\"strong\">" . get_city_displayname($city) . "</span>";
    $n = $city->getName();
    $i = $city->getId();
    echo " " . get_small_link("Areas", "/area/in/city/%s/%d", $n, $i);
    echo " " . get_small_link("Contacts", "/contact/in/city/%s/%d", $n, $i);
    echo " " . get_small_link("Donations", "/donation/in/city/%s/%d", $n, $i);
    echo " " . get_small_link("Orders", "/order/in/city/%s/%d", $n, $i);
    if (check_admin(1)) {
      echo " " . $city->getDeleteLink();
    }

    $q = new AreaQuery;
    $areas = $q->filterByCityId($city->getId())->find();
    if (count($areas)) {
      foreach ($areas as $area) {
        echo "<br>\nArea: " . $area->getLink();
      }
    }

    echo "</p>\n";
  }

  list($name, $id, $args) = parse_parameters($parameters);
  //echo "<p>$name($id) " . print_r($args, true) . "</p>\n";
  if (count($args)) {
    switch ($args[0]) {
      case "delete":
        confirm_delete_city($name, $id);
      break;

      case "confirmdelete":
        delete_city($name, $id);
      break;

      default:
        show_cities($name);
        show_new_city_form();
      break;
    }
  }
  else if (isset($name)) show_city($name, $id);
  else show_cities();

  if (count($parameters)) echo "<p class=\"noprint standout\">Show all <a href=\"/city\">cities</a></p>\n";
  show_new_city_form();

?>
