<?php

  if (isset($_POST['show_date'])) {
    header(sprintf("Location: http%s://%s/%s/date/%s", ($_SERVER['HTTPS']) ? "s" : "", $_SERVER['HTTP_HOST'], $module, $_POST['date']));
    exit;
  }

  /* Find orders scheduled for delivery on a certain date. */
  function get_orders_for_date($date) {
    $order_ids = array();

    /* $date is in Y-m-d format. */
    list($y, $m, $d) = explode('-', $date);
    $then = mktime(0, 0, 0, $m, $d, $y);

    $q = new OrderQuery();
    $orders = $q->filterByDate($then)->find();
    if (count($orders)) {
      foreach ($orders as $order) $order_ids[] = $order->getId();
    }
    else echo "<p>No deliveries for $date.</p>\n";

    return $order_ids;
  }

  /* Find orders scheduled for delivery today. */
  function get_orders_for_today() {
    return get_orders_for_date(date('Y-m-d'), time());
  }

  /* Find drivers with deliveries today. */
  function get_drivers_by_order_id($order_ids) {
    $driver_ids = array();

    if (count($order_ids)) {
      $dbh = Propel::getConnection();
      $sth = $dbh->prepare("select * from OrderState o where updated=(select max(updated) from OrderState where order_id=o.order_id) and order_id in (" . implode(",", $order_ids) . ") and driver_id is not null");
      $sth->execute();
      $order_states = OrderStatePeer::populateObjects($sth);
      if (count($order_states)) {
        foreach ($order_states as $order_state) $driver_ids[] = $order_state->getDriverId();
      }
      else echo "<p>No drivers assigned for deliveries.</p>\n";
    }

    return $driver_ids;
  }

  /* Find schedule for a driver today. */
  function get_driver_schedule_by_order_id($driver_id, $all_order_ids) {
    $order_ids = array();

    if (! count($all_order_ids)) {
      echo "<p>No orders for today.</p>\n";
      return null;
    }

    $dbh = Propel::getConnection();
    $sth = $dbh->prepare("select * from OrderState o where updated=(select max(updated) from OrderState where order_id=o.order_id) and order_id in (" . implode(",", $all_order_ids) . ") and driver_id=$driver_id");
    $sth->execute();
    $order_states = OrderStatePeer::populateObjects($sth);
    if (count($order_states)) {
      foreach ($order_states as $order_state) $order_ids[] = $order_state->getOrderId();
    }
    else echo "<p>No deliveries for this driver.</p>\n";

    return $order_ids;
  }

  function show_driver_forms($driver_ids, $date = null) {
    global $module;

    if (! count($driver_ids)) return;

    $q = new ContactQuery();
    $contacts = $q->filterById($driver_ids)->find();
    if (! count($contacts)) {
      echo "<p>Can't find drivers!</p>\n";
      return;
    }

    echo "<p>Drivers with deliveries scheduled on ";
    if ($date) echo $date;
    else echo date('Y-m-d', time());
    echo ":";
    foreach ($contacts as $contact) {
      printf("<br>\n<a href=\"/$module/%sdriver/%s/%d\">%s</a>", ($date) ? "date/$date/" : "", urlencode($contact->getDisplayname()), $contact->getId(), htmlspecialchars($contact->getDisplayname()));
    }
  }

  function show_driver_schedule($driver_name = null, $driver_id = null, $date = null) {
    if (isset($driver_id)) $contact = get_contact_by_id($driver_id);
    else if (isset($driver_name)) $contact = get_contact_by_name($driver_name);
    if (! $contact) {
      echo "<p>No such driver!</p>\n";
      return;
    }

    echo "<h3>Delivery schedule for <strong>" . htmlspecialchars($contact->getDisplayname()) . "</strong> on ";
    if ($date) {
      $order_ids = get_orders_for_date($date);
      echo $date;
    }
    else {
      $order_ids = get_orders_for_today();
      echo date('Y-m-d', time());
    }
    echo "</h3>\n";

    $q = new OrderQuery;
    $orders = $q->filterById(get_driver_schedule_by_order_id($contact->getId(), $order_ids))->find();

    /*
      Only allow drag and drop if there's more than one order otherwise IE will
      mess up the CSS.
    */
    $count = count($orders);
    if ($count) {
      foreach ($orders as $order) {
        if (! get_contact_by_id($order->getBeneficiaryId())) $count--;
      }
    }

    if ($count > 1) {
      echo "<div class=\"sortable\">\n";
      echo "<p class=\"noprint\">Drag delivery details to reorder the schedule.</p>\n";
    }

    foreach ($orders as $order) {
      $contact = get_contact_by_id($order->getBeneficiaryId());
      if (! $contact) continue;

      echo "<span>\n";

      $phones = array();
      $area = get_contact_area($contact);
      echo "<p>Order of <em>" . get_order_parcel_string($order) . "</em> for <strong>" . htmlspecialchars($contact->getDisplayname()) . "</strong> in " . htmlspecialchars(get_area_displayname($area)) . ".</p>\n";
      $hub = get_hub_by_id($order->getHubId(), false);
      if ($hub) {
        echo "<p>Deliver to hub <strong> " . htmlspecialchars($hub->getName()) . "</strong>";
        $address = get_hub_address($hub);
        $phone = $hub->getTelephone1();
        if ($phone) $phones[] = $phone;
        $phone = $hub->getTelephone2();
        if ($phone) $phones[] = $phone;
      }
      else {
        echo "<p>Deliver direct to beneficiary";
        $address = get_contact_address($contact);
        $phone = $contact->getTelephone1();
        if ($phone) $phones[] = $phone;
        $phone = $contact->getTelephone2();
        if ($phone) $phones[] = $phone;
      }
      $area = get_address_area($address);

      echo " in " . htmlspecialchars($area->getName()) . " at:<br>";
      $city = get_area_city($area);
      echo "\n<br>" . htmlspecialchars($address->getLine());
      echo "\n<br>" . htmlspecialchars($city->getName());
      echo "\n<br>" . htmlspecialchars($address->getPostcode());
      if (count($phones)) echo "\n<br><br>Telephone <strong>" . implode(" or ", $phones) . "</strong>";
      echo "</p>\n";

      $notes = $order->getNotes();
      if ($notes) {
        echo "<p><strong>Notes:</strong>\n";
        echo htmlspecialchars($notes);
        echo "</p>\n";
      }

      echo "<hr>\n";
      echo "</span>\n\n";
    }

    if ($count > 1) echo "</div>\n";
  }

  function show_delivery_date_form($date = null) {
    echo "<form method=\"POST\" action=\"" . $_SERVER['REQUEST_URI'] . "\">\n";
    echo "<p>Show deliveries for\n";
    show_date_form("date", $date);
    submit("show_date", "Show");
    echo "</form>\n";
  }

  $date = null;
  list($ignored, $id, $args) = parse_parameters($parameters);
  if ($parameters[0] == "date") {
    if ($args[0]) {
      if (preg_match('/^[1-9][0-9][0-9][0-9]-[01][0-9]-[0-3][0-9]$/', $args[0])) $date = $args[0];
      array_shift($args);
    }
  }

  if ($args[0] == "driver") array_shift($args);
  if (count($args)) show_driver_schedule($args[0], $args[1], $date);
  else {
    if ($date) $order_ids = get_orders_for_date($date);
    else $order_ids = get_orders_for_today();

    if ($order_ids) $driver_ids = get_drivers_by_order_id($order_ids);
    if ($driver_ids) show_driver_forms($driver_ids, $date);
    show_delivery_date_form($date);
  }

?>
